<?php

declare(strict_types=1);

namespace Drupal\cutoff_date\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\token\TreeBuilderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Cutoff Date configuration form.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * Entity type bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected EntityTypeBundleInfoInterface $bundleInfo;

  /**
   * Token tree builder service.
   *
   * @var \Drupal\token\TreeBuilderInterface
   */
  protected TreeBuilderInterface $tokenTreeBuilder;

  /**
   * Language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * Class constructor.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    TypedConfigManagerInterface $typed_config_manager,
    EntityTypeBundleInfoInterface $bundle_info,
    TreeBuilderInterface $token_tree_builder,
    LanguageManagerInterface $language_manager,
  ) {
    parent::__construct($config_factory, $typed_config_manager);
    $this->bundleInfo = $bundle_info;
    $this->tokenTreeBuilder = $token_tree_builder;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new self(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('entity_type.bundle.info'),
      $container->get('token.tree_builder'),
      $container->get('language_manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'cutoff_date_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['cutoff_date.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {

    $form['langcode'] = [
      '#type' => 'value',
      '#value' => $this->languageManager->getCurrentLanguage()->getId(),
      '#config_target' => 'cutoff_date.settings:langcode',
    ];

    $ts = $this->config('cutoff_date.settings')->get('timestamp') ?? time();
    $tz = $this->config('system.date')->get('timezone')['default'] ?? 'UTC';
    $default = DrupalDateTime::createFromTimestamp($ts);
    $form['timestamp'] = [
      '#type' => 'datetime',
      '#title' => $this->t('Global Cutoff Date (@tz)', ['@tz' => $tz]),
      '#description' => $this->t('Cutoff date in site default timezone. Once set, the cutoff date value is available via <strong>[cutoff_date:date]</strong> tokens, and can be used anywhere in the site configuration where tokens are accepted. See all the available tokens below.'),
      '#date_timezone' => $tz,
      '#date_increment' => 1,
      '#date_date_format' => 'Y-m-d',
      '#date_date_element' => 'date',
      '#date_time_format' => 'h:m',
      '#date_time_element' => 'time',
      '#default_value' => $default,
      '#config_target' => 'cutoff_date.settings:timestamp',
    ];

    $tree = $this->tokenTreeBuilder->buildRenderable(['cutoff_date'], [
      'click_insert' => FALSE,
      'show_restricted' => TRUE,
      'show_nested' => TRUE,
      'global_types' => TRUE,
      'recursion_limit' => 3,
    ]);
    $tree['#token_tree'] = ['cutoff_date' => $tree['#token_tree']['cutoff_date']];
    $form['tokens'] = $tree;

    $form['banner'] = [
      '#type' => 'details',
      '#title' => $this->t("Banner options"),
      '#open' => TRUE,
      '#tree' => FALSE,
    ];
    $form['banner']['show_banner'] = [
      '#type' => 'checkbox',
      '#description' => $this->t('Enable Cutoff Date banner on selected content types'),
      '#title' => $this->t('Show banner'),
      '#config_target' => 'cutoff_date.settings:show_banner',
    ];

    // #config_target cannot be used here due to a core issue.
    // @see https://www.drupal.org/project/drupal/issues/3463868
    $banner_text = $this->config('cutoff_date.settings')->get('banner_text') ?? [];
    $form['banner']['banner_text'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Banner text'),
      '#format' => $banner_text['format'] ?? '',
      '#default_value' => $banner_text['value'] ?? '',
    ];

    $form['banner']['banner_type'] = [
      '#type' => 'radios',
      '#options' => [
        'status' => $this->t('Status'),
        'error' => $this->t('Error'),
        'warning' => $this->t('Warning'),
      ],
      '#pretty_option' => TRUE,
      '#title' => $this->t('Banner type'),
      '#default_value' => $this->config('cutoff_date.settings')->get('banner_type') ?? 'warning',
      '#config_target' => 'cutoff_date.settings:banner_type',
    ];

    $definitions = $this->bundleInfo->getBundleInfo('node');
    array_walk($definitions, static function (&$value) {
      $value = $value['label'];
    });
    $form['banner']['content_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Content types'),
      '#description' => $this->t('Select content types on which the banner will be shown. Leave empty for all content types.'),
      '#pretty_option' => TRUE,
      '#options' => $definitions,
      '#config_target' => 'cutoff_date.settings:content_types',
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $date = $form_state->getValue('timestamp');
    if (!empty($date)) {
      $form_state->setValue('timestamp', $date->getTimestamp());
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Text format values must be stored manually, since #config_target does
    // not work with text_format elements.
    // @see https://www.drupal.org/project/drupal/issues/3463868
    if ($form_state->hasValue('banner_text')) {
      $banner_text = $form_state->getValue('banner_text');
      $this->config('cutoff_date.settings')->set('banner_text', $banner_text);
    }

    parent::submitForm($form, $form_state);
  }

}
